#include "bonus_points.hpp"

#include <algorithm>
#include <ranges>
#include <stdexcept>
#include <string>

BonusPoints::BonusPoints(const CsvDump &c) noexcept(false) : mBonusPoints(c) {
    if (c.data.empty())
        throw std::invalid_argument("Read bonus-CSV: Empty file");

    const auto nrRows = c.data.size();

    // No data aside from header is allowed
    if (nrRows < 2)
        return;

    mRegistrationNr.reserve(static_cast<int>(nrRows) - 1);
    mNrBonusPoints.reserve(static_cast<int>(nrRows) - 1);

    // Get column index
    const auto registrationNrColumn = static_cast<std::size_t>(
        std::ranges::find(c.data[0], "Matrikelnummer") -
        std::ranges::begin(c.data[0]));
    const auto bonusPointColumn = static_cast<std::size_t>(
        std::ranges::find(c.data[0], "Bonuspunkte (Punkte)") -
        std::ranges::begin(c.data[0]));

    // Throw if relevant columns not found
    if (registrationNrColumn >= c.data[0].size())
        throw std::invalid_argument(
            "Read bonus-CSV: No registration nr column found");
    if (bonusPointColumn >= c.data[0].size())
        throw std::invalid_argument(
            "Read bonus-CSV: No bonus point column found");

    // Insert relevant data into lists
    for (std::size_t i = 1; i < nrRows; ++i) {
        // Registration number column sometimes contains an email address
        // instead of a registration number. Todo: Try to match them up via name
        try {
            mRegistrationNr.emplace_back(
                std::stoi(c.data[i][registrationNrColumn]));
        } catch (...) {
            continue;
        }
        mNrBonusPoints.emplace_back(
            stringToPoints(c.data[i][bonusPointColumn]));
    }
}
