#pragma once

#include "changed_max_points.hpp"
#include "node.hpp"

#include "util/enums.hpp"
#include "util/grade_boundary.hpp"
#include "util/points.hpp"

#include <QList>

struct FirstTry;
struct UserPointConfig;

namespace impl {
[[nodiscard]] Points
calculateAbsolutePassingLimit(GradeBoundary max,
                              GradeBoundaryStepSize stepSize) noexcept;

[[nodiscard]] Points calculateRelativePassingLimit(
    GradeBoundary max, GradeBoundaryStepSize stepSize,
    const QList<Points> &firstTryClosedPoints) noexcept;

[[nodiscard]] Points
calculatePassingLimit(GradeBoundary max, GradeBoundaryStepSize stepSize,
                      const QList<Points> &firstTryClosedPoints) noexcept;
} // namespace impl

template <GradingType Grading>
struct PassingLimit;

template <>
struct PassingLimit<GradingType::Closed>
    : Node<ChangedMaxPoints<GradingType::Closed>, FirstTry, UserPointConfig> {
    using Node<ChangedMaxPoints<GradingType::Closed>, FirstTry,
               UserPointConfig>::Node;

    void recalculate();

    // Starting value for the passing limit (4.0 grade boundary)
    GradeBoundary mPassingLimit;

    // Max value for the passing limit
    GradeBoundary mMaxPassingLimit;

private:
    Points mRawPassingLimit;
};

template <>
struct PassingLimit<GradingType::Open>
    : Node<ChangedMaxPoints<GradingType::Open>, UserPointConfig> {
    using Node<ChangedMaxPoints<GradingType::Open>, UserPointConfig>::Node;

    void recalculate();

    // Starting value for the passing limit (4.0 grade boundary)
    GradeBoundary mPassingLimit;

    // Max value for the passing limit
    GradeBoundary mMaxPassingLimit;

private:
    // Passing limit as points, instead of as grade boundary
    Points mRawPassingLimit;
};
