#pragma once

#include <compare>
#include <concepts>
#include <string>

struct Points {
    double p = 0.;

    [[nodiscard]] auto operator<=>(const Points &) const noexcept = default;

    constexpr Points &operator+=(const Points &rhs) noexcept {
        p += rhs.p;
        return *this;
    }
    constexpr Points &operator-=(const Points &rhs) noexcept {
        p -= rhs.p;
        return *this;
    }
};

[[nodiscard]] constexpr Points operator+(Points a, const Points &b) noexcept {
    a += b;
    return a;
}
[[nodiscard]] constexpr Points operator-(Points a, const Points &b) noexcept {
    a -= b;
    return a;
}

template <std::integral Int>
[[nodiscard]] constexpr Points operator/(const Points &a,
                                         const Int b) noexcept {
    return Points{a.p / static_cast<double>(b)};
}

[[nodiscard]] constexpr double operator/(const Points &a,
                                         const Points &b) noexcept {
    return a.p / b.p;
}

template <std::floating_point Float>
[[nodiscard]] constexpr Points operator*(const Points &a,
                                         const Float b) noexcept {
    return Points{a.p * static_cast<double>(b)};
}

/**
 * \brief Call std::stod, but interpret an empty string as 0.
 */
[[nodiscard]] Points stringToPoints(std::string s) noexcept(false);
