#pragma once

#include "data/nodes/dynexite_exam.hpp"

#include "util/enums.hpp"
#include "util/grade_boundary.hpp"
#include "util/grade_boundary_step.hpp"
#include "util/storage_helper.hpp"

// #include "view/gui/custom_double_spin_box.hpp"
#include "view/gui/custom_slider.hpp"

#include <QCheckBox>
#include <QDoubleSpinBox>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QList>
#include <QString>
#include <QVBoxLayout>
#include <QWidget>

class MinMaxPointScrewColumn : public QVBoxLayout {
    Q_OBJECT

public:
    explicit MinMaxPointScrewColumn(GradingType grading,
                                    QWidget *parent = nullptr);

    void setWidgetsEnabled(bool enable) const;

private:
    QWidget *mParent;

    GradingType mGrading;

    QHBoxLayout *mHLayouts[4]{};
    QLabel *mDescriptions[2]{};
    QDoubleSpinBox *mSpinBoxes[4]{};
    CustomSlider *mSliders[2]{};

    void setSpinboxRange(int row, GradeBoundary min, GradeBoundary max,
                         GradeBoundary start,
                         GradeBoundaryStepSize minPointStep) const;
    void setSliderRange(int row, int sliderIdx, GradeBoundary rangeMin,
                        GradeBoundary rangeMax, GradeBoundary start,
                        GradeBoundary maxPoints) const;

    void createLabel(GradingType grading, int labelIdx, const QString &text,
                     int row, QWidget *parent);
    void createSlider(GradingType grading, int sliderIdx, int row,
                      QWidget *parent);

private slots:
    void gradesChanged();
    void sliderChange(GradingType grading, int sliderIdx, int value);
    void spinboxChange(GradingType grading, int spinboxIdx, double value);
};

class MinMaxPointScrews : public QWidget {
    Q_OBJECT

public:
    explicit MinMaxPointScrews(QWidget *parent = nullptr);

    void setEnabled(GradingType type, bool enabled) const;

private slots:
    void gradesChanged();

private:
    QVBoxLayout mVLayout{this};
    QHBoxLayout mHLayout{};
    QGridLayout mAvgLayout{};

    MinMaxPointScrewColumn *mClosedColumn = nullptr;
    MinMaxPointScrewColumn *mOpenColumn = nullptr;

    QLabel mAverageGrade{}, mAverageGradePassed{};

    void createScrewColumns();
    void createAvgLabels();
};
