#include "data/nodes/first_try.hpp"

#include "data/testable_objects.hpp"

#include "data/graph.hpp"

#include <gtest/gtest.h>

// Execute for each DAG
class FirstTryDagTest
    : public testing::TestWithParam<std::pair<std::string, DAG>> {};

INSTANTIATE_TEST_SUITE_P(FirstTryDagParameterizedTests, FirstTryDagTest,
                         testing::ValuesIn(exams()));

// Check the results for our known test files
TEST_P(FirstTryDagTest, CorrectlyCalculatedInDag) {
    const auto &name = GetParam().first;
    const auto &graph = GetParam().second;

    if (name == "split") {
        ASSERT_EQ(graph.mFirstTry.mZerothTryClosedPoints.size(), 1u);
        EXPECT_EQ(graph.mFirstTry.mZerothTryClosedPoints[0].p, 81.);

        EXPECT_EQ(graph.mFirstTry.mFirstTryClosedPoints.size(), 23u);
    } else if (name == "closed") {
        EXPECT_EQ(graph.mFirstTry.mZerothTryClosedPoints.size(), 0u);

        EXPECT_EQ(graph.mFirstTry.mFirstTryClosedPoints.size(), 38u);

    } else if (name == "closed_bonus") {
        ASSERT_EQ(graph.mFirstTry.mZerothTryClosedPoints.size(), 1u);
        EXPECT_EQ(graph.mFirstTry.mZerothTryClosedPoints[0].p, 204.);

        EXPECT_LE(graph.mFirstTry.mFirstTryClosedPoints.size(), 585u);
        EXPECT_GE(graph.mFirstTry.mFirstTryClosedPoints.size(), 552u);

    } else {
        FAIL() << "Input file with unknown name " << name;
    }
}
