#include "file_io/csv_parser.hpp"

#include "gtest/gtest.h"
#include <filesystem>

using std::filesystem::directory_iterator;

TEST(CsvParserTest, parseDynexite) {
    try {
        for (const auto &it : {directory_iterator(std::filesystem::path(
                                   TESTING_DYNEXITE_DIRECTORY)),
                               directory_iterator(std::filesystem::path(
                                   TESTING_RWTHONLINE_DIRECTORY))}) {
            for (const auto &f : it) {

                SCOPED_TRACE("Reading in file " + f.path().string());

                // expect only .csv and .gitkeep files in the repository
                if (f.path().extension() ==
                    "") { // .gitkeep counts as empty extension
                    continue;
                }
                if (f.path().extension() != ".csv") {
                    FAIL() << "Unknown file extension: "
                           << f.path().extension();
                }
                try {
                    auto p = readCsv(f);
                    EXPECT_TRUE(!!p) << "readCsv created a NULL-pointer";
                } catch (const std::exception &e) {
                    FAIL() << "readCsv threw an exception: " << e.what();
                }
            }
        }
    } catch (const std::exception &e) {

        FAIL()
            << "directory_iterator threw an exception(" << e.what()
            << "); ensure that TESTING_DYNEXITE_DIRECTORY is set up correctly";
    }
}
